<?php

require_once('MailchimpRequest.php');
require_once('MailchimpResponse.php');

class MailchimpClient
{
    /**
     * Endpoint url
     * @var string
     */
    public $url = '';

    /**
     * The main request object
     * @var MailchimpRequest
     */
    protected $request;

    /**
     * Mailchimp API key
     * @var string
     */
    protected $apiKey;

    /**
     * Mailchimp API Endpoint
     * @var string
     */
    protected $apiEndpoint = 'https://%dc%.api.mailchimp.com/3.0/';

    /**
     * Array of headers
     * @var Array
     */
    protected $headers = array();

    /**
     * Hold the errors
     * @var string
     */
    private $error = '';

    /**
     * Associative array of all registered hooks
     * To add your hooks, extend this class and call register to register a new hook
     * @var array
     */
    private $hooks = array(
        'subscribe' => array('post', 'lists/%s/members'),
        'unsubscribe' => array('patch', 'lists/%s/members/%s'),
        'status' => array('patch', 'lists/%s/members/%s'),
        'remove' => array('delete', 'lists/%s/members/%s'),
        'lists' => array('get', 'lists/')
    );

    /**
     * Construct the mailchimp client
     * @method __construct
     * @param  string      $apiKey mailchimp api key (https://us12.admin.mailchimp.com/account/api/)
     */
    public function __construct($apiKey)
    {
        $this->apiKey = $apiKey;

        list(, $dc) = explode('-', $this->apiKey);
        $this->apiEndpoint = preg_replace('/%dc%/', $dc, $this->apiEndpoint);
        $this->request = new MailchimpRequest($this->apiEndpoint,
            array(
                'Accept' => 'application/vnd.api+json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Basic '.base64_encode( 'user:'. $this->apiKey ),
                'User-Agent' => isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT']: 'Mozilla/5.0',
            )
        );
    }

    /**
     * Register your custom hooks
     * @method register
     * @param  string   $hook   [description]
     * @param  string   $method the method you need to request upon, valid methods are 'get', 'put', 'patch', 'post', 'delete'
     * @param  string   $uri    the uri of the endpoint excluding the base URL, ex: 'lists/%s/members'
     * @return void
     */
    public function register($hook, $method, $uri)
    {
        $this->hooks[$hook] = array($method, $uri);
    }

    /**
     * Get all lists
     * @method lists
     * @return json all mailchimp lists of the current account
     */
    public function lists()
    {
        return $this->call('lists');
    }

    /**
     * Subscribe to a mailchimp list
     * @method subscribe
     * @param  string    $listId      the list id
     * @param  string    $email       the subscriber email
     * @param  boolean   $doubleOptin whether to use double option or single option
     * @param  array     $mergeVars   merge variables
     * @return json                   mailchimp response
     *
     * You must call status method to make sure the subscriber is not already subscribed before calling this method
     */
    public function subscribe($listId, $email, $doubleOptin = true, $mergeVars = array())
    {
        $requestData = array(
            'email_address' => $email,
            'status' => $doubleOptin ? 'pending' : 'subscribed',
        );
        if (count($mergeVars) > 0) {
            $requestData['merge_fields'] = $mergeVars;
        }
        return $this->call('subscribe', array($listId), $requestData);
    }

    /**
     * Unsubscribe from a mailchimp list
     * @method unsubscribe
     * @param  string      $listId the list id
     * @param  string      $email  the subscribed email
     * @return json                mailchimp response
     */
    public function unsubscribe($listId, $email)
    {
        $email = strtolower($email);
        $hash = md5($email);
        return $this->call('unsubscribe', array($listId, $hash), array('status' => 'unsubscribed'));
    }

    /**
     * Get the status of the current subscriber
     * @method status
     * @param  string $listId the list id
     * @param  string $email  the new/subscribed email
     * @return json           mailchimp response
     */
    public function status($listId, $email)
    {
        $email = strtolower($email);
        $hash = md5($email);
        return $this->call('status', array($listId, $hash));
    }

    /**
     * Remove the subscriber from a list
     * @method remove
     * @param  string $listId the list id
     * @param  string $email  the new/subscribed email
     * @return json           mailchimp response
     */
    public function remove($listId, $email)
    {
        $email = strtolower($email);
        $hash = md5($email);
        return $this->call('remove', array($listId, $hash));
    }

    /**
     * Caller function to call the request wrapper to send a curl request to mailchimp api v3
     * @method call
     * @param  string $hook The hook, it must be registered in the array of hooks
     * @param  array  $main the main args that needs to be sent along with the url
     * @param  array  $args the submission args and data that needs to be sent to mailchimp
     * @return json   mailchimp json response
     */
    protected function call($hook, $main = array(), $args = array())
    {
        if($hook == '' || !isset($this->hooks[$hook])){
            throw new Exception('Hook doesn\t exist, Please use $client->register to register your hook');
        }
        //$args['apikey'] = $this->apiKey;
        $response = null;

        try {
            $obj = $this->hooks[$hook];
            $uri = vsprintf($obj[1], $main);
            $response = $this->request->{$obj[0]}($uri, $args);
            return $response;
        } catch (MailchimpRequestException $e) {
            return null;
        }
    }
}
